
import math

import numpy as np
import pytest

from barycentric import (calc_barycentric_weights, create_verts, interpolate_z,
                         point_in_triangle, random_weights)

TRIALS = 10_000

TEST_BOUNDS = [
    (((0, 1), (0, 1), (0, 1))),
    (((0, 10), (0, 10), (0, 10))),
    (((0, 100), (0, 100), (10, 20))),
    (((100_000, 100_100), (3_000_000, 3_000_100), (100, 200))),
    (((0, 1), (0, 1), (100, 200), (-1000, -1500))),
]


@pytest.mark.parametrize("n", range(2, 6))
def test_random_weights(n, trials=TRIALS):
    for _ in range(trials):
        weights = random_weights(n)
        assert len(weights) == n
        assert math.isclose(np.sum(weights), 1)


@pytest.mark.parametrize("bounds", TEST_BOUNDS)
def test_interpolation(bounds, trials=TRIALS):
    for _ in range(trials):
        t = create_verts(bounds=bounds)
        # w = np.repeat(1/3, 3) # center
        w = random_weights()
        p = point_in_triangle(t, w)
        z = interpolate_z(t, p)
        assert np.isclose(p[2:], z[0], rtol=1e-6).all()


@pytest.mark.parametrize("bounds", TEST_BOUNDS)
def test_calc_barycentric_weights(bounds, trials=TRIALS):
    for _ in range(trials):
        t = create_verts(bounds=bounds)
        w = random_weights()
        p = point_in_triangle(t, w)
        calc_w = calc_barycentric_weights(t, p)
        assert np.isclose(w, calc_w).all()
